#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright(C) 2013 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import Unity
from gi.repository import Gio, GLib
import gettext
import os.path
import shutil
import sqlite3

APP_NAME = 'unity-scope-zotero'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Academic.Zotero'
UNIQUE_PATH = '/com/canonical/unity/scope/academic/zotero'
SEARCH_URI = ''
SEARCH_HINT = _('Search Zotero')
NO_RESULTS_HINT = _('Sorry, there are no Zotero results that match your search.')
PROVIDER_CREDITS = _('Powered by Zotero')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR + 'service-zotero.svg'
DEFAULT_RESULT_ICON = SVG_DIR + 'result-academic.svg'
DEFAULT_RESULT_MIMETYPE = 'text/html'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT

PATHS = [os.getenv("HOME") + "/.mozilla/firefox/", os.getenv("HOME") + "/.zotero/zotero/"]
QUERY = ''' SELECT title.value, url.value, pub.value, c1.lastName, c2.lastName, c3.lastName, c4.lastName, c5.lastName, abstract.value
FROM items i
INNER JOIN itemDataValues title ON title.valueID =(SELECT itemData.valueID FROM itemData WHERE itemData.fieldID =(SELECT fieldID FROM fields WHERE fields.fieldName = 'title' LIMIT 1) AND itemData.itemID=i.itemID LIMIT 1)
LEFT JOIN itemTypes t ON t.itemTypeID = i.itemTypeID
LEFT JOIN itemDataValues url ON url.valueID =(SELECT itemData.valueID FROM itemData WHERE itemData.fieldID =(SELECT fieldID FROM fields WHERE fields.fieldName = 'url' LIMIT 1) AND itemData.itemID=i.itemID LIMIT 1)
LEFT JOIN creatorData c1 ON c1.creatorDataID =(SELECT creatorDataID FROM creators WHERE creatorID =(SELECT creatorID FROM itemCreators WHERE itemID = i.itemID ORDER BY orderIndex LIMIT 0,1) LIMIT 1)
LEFT JOIN creatorData c2 ON c2.creatorDataID =(SELECT creatorDataID FROM creators WHERE creatorID =(SELECT creatorID FROM itemCreators WHERE itemID = i.itemID ORDER BY orderIndex LIMIT 1,1) LIMIT 1)
LEFT JOIN creatorData c3 ON c3.creatorDataID =(SELECT creatorDataID FROM creators WHERE creatorID =(SELECT creatorID FROM itemCreators WHERE itemID = i.itemID ORDER BY orderIndex LIMIT 2,1) LIMIT 1)
LEFT JOIN creatorData c4 ON c4.creatorDataID =(SELECT creatorDataID FROM creators WHERE creatorID =(SELECT creatorID FROM itemCreators WHERE itemID = i.itemID ORDER BY orderIndex LIMIT 3,1) LIMIT 1)
LEFT JOIN creatorData c5 ON c5.creatorDataID =(SELECT creatorDataID FROM creators WHERE creatorID =(SELECT creatorID FROM itemCreators WHERE itemID = i.itemID ORDER BY orderIndex LIMIT 4,1) LIMIT 1)
LEFT JOIN itemDataValues pub ON pub.valueID =(SELECT itemData.valueID FROM itemData WHERE itemData.fieldID =(SELECT fieldID FROM fields WHERE fields.fieldName = 'publicationTitle' LIMIT 1) AND itemData.itemID=i.itemID LIMIT 1)
LEFT JOIN itemDataValues abstract ON abstract.valueID =(SELECT itemData.valueID FROM itemData WHERE itemData.fieldID =(SELECT fieldID FROM fields WHERE fields.fieldName = 'abstractNote' LIMIT 1) AND itemData.itemID=i.itemID LIMIT 1)
LEFT JOIN deletedItems ON i.itemID = deletedItems.itemID
WHERE deletedItems.itemID IS NULL AND t.typeName = "journalArticle"'''

c1 = {'id': 'scholar',
      'name': _('Articles'),
      'icon': SVG_DIR + 'group-installed.svg',
      'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]

FILTERS = []

EXTRA_METADATA = []


def search(search, filters):
    '''
    Any search method returning results as a list of tuples.
    Available tuple fields:
    uri(string)
    icon(string)
    title(string)
    comment(string)
    dnd_uri(string)
    mimetype(string)
    category(int)
    result_type(Unity ResultType)
    extras metadata fields(variant)
    '''
    results = []
    icon_hint = Gio.ThemedIcon.new("text-html").to_string()

    dbFile = ""
    for path in PATHS:
        for folder in [default for default in os.listdir(path) if default.endswith(".default")]:
            dbFile = path + folder + "/zotero/zotero.sqlite"

        if os.path.exists(dbFile):
            backup_dbFile = path + folder + "/zotero/zotero-lensbackup.sqlite"
            shutil.copy2(dbFile, backup_dbFile)

        records = []
        if os.path.exists(backup_dbFile):
            conn = sqlite3.connect(backup_dbFile)
            c = conn.cursor()
            rows = c.execute(QUERY)
            records = c.fetchall()
            c.close()
        for record in records:
            title = record[0]
            if len(title) > 70:
                shorttitle = title[:69] + u"…"
            uri = record[1]
            journal = record[2]
            author1 = record[3]
            author2 = record[4]
            author3 = record[5]
            author4 = record[6]
            author5 = record[7]
            authors = "%s, %s, %s, %s, %s" % (author1, author2, author3, author4, author5)
            abstract = record[8]
            if len(authors) > 30:
                authors = authors[:26] + " et al"
            icon_hint = Gio.ThemedIcon.new('text-html').to_string()
            if search.lower() in title.lower():
                if uri:
                    results.append({'uri': uri,
                                    'icon': icon_hint,
                                    'category': 0,
                                    'title': shorttitle,
                                    'comment': '%s\n%s\n%s\n%s' % (authors, journal, title, abstract)})
    return results


def preview(scope, uri):
    """Preview request handler"""
    model = scope.props.results_model
    iteration = model.get_first_iter()
    end_iter = model.get_last_iter()
    preview = None
    while iteration != end_iter:
        if model.get_value(iteration, 0) == uri:
            title = ""
            description = ""
            preview_info = model.get_value(iteration, 5)
            try:
                description += "<big><b>" + preview_info.split("\n")[2].title() + "</b></big>\n\n"
            except:
                print("No Title")

            try:
                description += "<b>" + preview_info.split("\n")[1].title() + "</b>\n\n"
            except:
                print("No Journal")

            try:
                description += "<b>" + preview_info.split("\n")[0].title() + "</b>\n\n"
            except:
                print("No Authors")

            try:
                description += preview_info.split("\n")[3].title()
            except:
                print("No Abstract")

            # Assemble the preview
            preview = Unity.GenericPreview.new(title, description, None)
            preview.props.image_source_uri = "file://" + model.get_value(iteration, 1)

            # Add the "View" action
            view_action = Unity.PreviewAction.new("activate_uri", "View Article", None)
            view_action.connect("activated", activate)
            preview.add_action(view_action)

            break

        iteration = model.next(iteration)
    if preview is None:
        print("Couldn't find model row for requested preview uri: '%s'", uri)
    return preview


def activate(scope, uri):
    return Unity.ActivationResponse(handled=Unity.HandledType.NOT_HANDLED)


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch(Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super(MySearch, self).__init__()
        self.set_search_context(search_context)

    def do_run(self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                i['metadata'] = {}
                if EXTRA_METADATA:
                    for e in i:
                        for m in EXTRA_METADATA:
                            if m['id'] == e:
                                i['metadata'][e] = i[e]
                i['metadata']['provider_credits'] = GLib.Variant('s', PROVIDER_CREDITS)
                result = Unity.ScopeResult.create(str(i['uri']), str(i['icon']),
                                                  i['category'], i['result_type'],
                                                  str(i['mimetype']), str(i['title']),
                                                  str(i['comment']), str(i['dnd_uri']),
                                                  i['metadata'])
                result_set.add_result(result)
        except Exception as error:
            print(error)


class Scope(Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint(self):
        return SEARCH_HINT

    def do_get_schema(self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories(self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new(c['id'], c['name'],
                                         Gio.ThemedIcon.new(c['icon']),
                                         c['renderer'])
                cs.add(cat)
        return cs

    def do_get_filters(self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new()
#        if FILTERS:
#
        return fs

    def do_get_group_name(self):
        return GROUP_NAME

    def do_get_unique_name(self):
        return UNIQUE_PATH

    def do_create_search_for_query(self, search_context):
        se = MySearch(search_context)
        return se


def load_scope():
    return Scope()
